@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2010 Jeffrey Stedfast
+ *  Copyright (C) 2000-2011 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -81,7 +81,13 @@ static const char *gpg_hash_name (GMimeCryptoContext *ctx, GMimeCryptoHash hash)
 static int gpg_sign (GMimeCryptoContext *ctx, const char *userid,
 		     GMimeCryptoHash hash, GMimeStream *istream,
 		     GMimeStream *ostream, GError **err);
-	
+
+static const char *gpg_get_signature_protocol (GMimeCryptoContext *ctx);
+
+static const char *gpg_get_encryption_protocol (GMimeCryptoContext *ctx);
+
+static const char *gpg_get_key_exchange_protocol (GMimeCryptoContext *ctx);
+
 static GMimeSignatureValidity *gpg_verify (GMimeCryptoContext *ctx, GMimeCryptoHash hash,
 					   GMimeStream *istream, GMimeStream *sigstream,
 					   GError **err);
@@ -90,8 +96,8 @@ static int gpg_encrypt (GMimeCryptoContext *ctx, gboolean sign, const char *user
 			GMimeCryptoHash hash, GPtrArray *recipients, GMimeStream *istream,
 			GMimeStream *ostream, GError **err);
 
-static GMimeSignatureValidity *gpg_decrypt (GMimeCryptoContext *ctx, GMimeStream *istream,
-					    GMimeStream *ostream, GError **err);
+static GMimeDecryptionResult *gpg_decrypt (GMimeCryptoContext *ctx, GMimeStream *istream,
+					   GMimeStream *ostream, GError **err);
 
 static int gpg_import_keys (GMimeCryptoContext *ctx, GMimeStream *istream,
 			    GError **err);
@@ -146,6 +152,9 @@ g_mime_gpg_context_class_init (GMimeGpgContextClass *klass)
 	crypto_class->decrypt = gpg_decrypt;
 	crypto_class->import_keys = gpg_import_keys;
 	crypto_class->export_keys = gpg_export_keys;
+	crypto_class->get_signature_protocol = gpg_get_signature_protocol;
+	crypto_class->get_encryption_protocol = gpg_get_encryption_protocol;
+	crypto_class->get_key_exchange_protocol = gpg_get_key_exchange_protocol;
 }
 
 static void
@@ -156,10 +165,6 @@ g_mime_gpg_context_init (GMimeGpgContext *ctx, GMimeGpgContextClass *klass)
 	ctx->auto_key_retrieve = FALSE;
 	ctx->always_trust = FALSE;
 	ctx->path = NULL;
-	
-	crypto->sign_protocol = "application/pgp-signature";
-	crypto->encrypt_protocol = "application/pgp-encrypted";
-	crypto->key_protocol = "application/pgp-keys";
 }
 
 static void
@@ -238,6 +243,24 @@ gpg_hash_name (GMimeCryptoContext *ctx, GMimeCryptoHash hash)
 	}
 }
 
+static const char *
+gpg_get_signature_protocol (GMimeCryptoContext *ctx)
+{
+	return "application/pgp-signature";
+}
+
+static const char *
+gpg_get_encryption_protocol (GMimeCryptoContext *ctx)
+{
+	return "application/pgp-encrypted";
+}
+
+static const char *
+gpg_get_key_exchange_protocol (GMimeCryptoContext *ctx)
+{
+	return "application/pgp-keys";
+}
+
 enum _GpgCtxMode {
 	GPG_CTX_MODE_SIGN,
 	GPG_CTX_MODE_VERIFY,
@@ -256,6 +279,7 @@ struct _GpgCtx {
 	
 	char *userid;
 	GPtrArray *recipients;
+	GMimeCryptoCipherAlgo cipher;
 	GMimeCryptoHash hash;
 	
 	int stdin_fd;
@@ -278,6 +302,9 @@ struct _GpgCtx {
 	GByteArray *diag;
 	GMimeStream *diagnostics;
 	
+	GMimeCryptoRecipient *encrypted_to;  /* full list of encrypted-to recipients */
+	GMimeCryptoRecipient *enc_to;        /* most recent recipient (tail pointer) */
+	
 	GMimeSigner *signers;
 	GMimeSigner *signer;
 	
@@ -318,6 +345,7 @@ gpg_ctx_new (GMimeGpgContext *ctx)
 	
 	gpg->userid = NULL;
 	gpg->recipients = NULL;
+	gpg->cipher = GMIME_CRYPTO_CIPHER_ALGO_DEFAULT;
 	gpg->hash = GMIME_CRYPTO_HASH_DEFAULT;
 	gpg->always_trust = FALSE;
 	gpg->armor = FALSE;
@@ -336,6 +364,9 @@ gpg_ctx_new (GMimeGpgContext *ctx)
 	gpg->need_passwd = FALSE;
 	gpg->need_id = NULL;
 	
+	gpg->encrypted_to = NULL;
+	gpg->enc_to = (GMimeCryptoRecipient *) &gpg->encrypted_to;
+	
 	gpg->signers = NULL;
 	gpg->signer = (GMimeSigner *) &gpg->signers;
 	
@@ -458,7 +489,8 @@ gpg_ctx_get_diagnostics (struct _GpgCtx *gpg)
 static void
 gpg_ctx_free (struct _GpgCtx *gpg)
 {
-	GMimeSigner *signer, *next;
+	GMimeCryptoRecipient *recipient, *nr;
+	GMimeSigner *signer, *ns;
 	guint i;
 	
 	g_hash_table_destroy (gpg->userid_hint);
@@ -498,11 +530,18 @@ gpg_ctx_free (struct _GpgCtx *gpg)
 	
 	g_object_unref (gpg->diagnostics);
 	
+	recipient = gpg->encrypted_to;
+	while (recipient != NULL) {
+		nr = recipient->next;
+		g_mime_crypto_recipient_free (recipient);
+		recipient = nr;
+	}
+	
 	signer = gpg->signers;
 	while (signer != NULL) {
-		next = signer->next;
+		ns = signer->next;
 		g_mime_signer_free (signer);
-		signer = next;
+		signer = ns;
 	}
 	
 	g_slice_free (struct _GpgCtx, gpg);
@@ -537,39 +576,6 @@ gpg_hash_str (GMimeCryptoHash hash)
 	}
 }
 
-static GMimeCryptoPubKeyAlgo
-gpg_pubkey_algo (unsigned long id)
-{
-	switch (id) {
-	case 1: return GMIME_CRYPTO_PUBKEY_ALGO_RSA;
-	case 2: return GMIME_CRYPTO_PUBKEY_ALGO_RSA_E;
-	case 3: return GMIME_CRYPTO_PUBKEY_ALGO_RSA_S;
-	case 16: return GMIME_CRYPTO_PUBKEY_ALGO_ELG_E;
-	case 17: return GMIME_CRYPTO_PUBKEY_ALGO_DSA;
-	case 20: return GMIME_CRYPTO_PUBKEY_ALGO_ELG;
-	default: return GMIME_CRYPTO_PUBKEY_ALGO_DEFAULT;
-	}
-}
-
-static GMimeCryptoHash
-gpg_hash_algo (unsigned long id)
-{
-	switch (id) {
-	case 1: return GMIME_CRYPTO_HASH_MD5;
-	case 2: return GMIME_CRYPTO_HASH_SHA1;
-	case 3:	return GMIME_CRYPTO_HASH_RIPEMD160;
-	case 5: return GMIME_CRYPTO_HASH_MD2;
-	case 6: return GMIME_CRYPTO_HASH_TIGER192;
-	case 7: return GMIME_CRYPTO_HASH_HAVAL5160;
-	case 8: return GMIME_CRYPTO_HASH_SHA256;
-	case 9: return GMIME_CRYPTO_HASH_SHA384;
-	case 10: return GMIME_CRYPTO_HASH_SHA512;
-	case 11: return GMIME_CRYPTO_HASH_SHA224;
-	case 301: return GMIME_CRYPTO_HASH_MD4;
-	default: return GMIME_CRYPTO_HASH_DEFAULT;
-	}
-}
-
 static char **
 gpg_ctx_get_argv (struct _GpgCtx *gpg, int status_fd, int secret_fd, char ***strv)
 {
@@ -823,37 +829,56 @@ next_token (char *in, char **token)
 	return inptr;
 }
 
+/**
+ * gpg_ctx_add_signer:
+ * @gpg: GnuPG context
+ * @status: a #GMimeSignerStatus
+ * @info: a string with the signer's info
+ *
+ * Parses GOODSIG, BADSIG, EXPSIG, EXPKEYSIG, and REVKEYSIG status messages
+ * into a newly allocated #GMimeSigner and adds it to @gpg's signer list.
+ *
+ * Returns: the newly added signer for the caller to add more info to.
+ **/
+static GMimeSigner *
+gpg_ctx_add_signer (struct _GpgCtx *gpg, GMimeSignerStatus status, char *info)
+{
+	GMimeSigner *signer;
+	
+	signer = g_mime_signer_new (status);
+	gpg->signer->next = signer;
+	gpg->signer = signer;
+	
+	/* get the key id of the signer */
+	info = next_token (info, &signer->keyid);
+	
+	/* the rest of the string is the signer's name */
+	signer->name = g_strdup (info);
+	
+	return signer;
+}
+
 static void
 gpg_ctx_parse_signer_info (struct _GpgCtx *gpg, char *status)
 {
 	GMimeSigner *signer;
+	char *inend;
 	
 	if (!strncmp (status, "SIG_ID ", 7)) {
 		/* not sure if this contains anything we care about... */
 	} else if (!strncmp (status, "GOODSIG ", 8)) {
-		status += 8;
-		
-		signer = g_mime_signer_new (GMIME_SIGNER_STATUS_GOOD);
-		gpg->signer->next = signer;
-		gpg->signer = signer;
-		
-		/* get the key id of the signer */
-		status = next_token (status, &signer->keyid);
-		
-		/* the rest of the string is the signer's name */
-		signer->name = g_strdup (status);
+		gpg_ctx_add_signer (gpg, GMIME_SIGNER_STATUS_GOOD, status + 8);
 	} else if (!strncmp (status, "BADSIG ", 7)) {
-		status += 7;
-		
-		signer = g_mime_signer_new (GMIME_SIGNER_STATUS_BAD);
-		gpg->signer->next = signer;
-		gpg->signer = signer;
-		
-		/* get the key id of the signer */
-		status = next_token (status, &signer->keyid);
-		
-		/* the rest of the string is the signer's name */
-		signer->name = g_strdup (status);
+		gpg_ctx_add_signer (gpg, GMIME_SIGNER_STATUS_BAD, status + 7);
+	} else if (!strncmp (status, "EXPSIG ", 7)) {
+		signer = gpg_ctx_add_signer (gpg, GMIME_SIGNER_STATUS_ERROR, status + 7);
+		signer->errors |= GMIME_SIGNER_ERROR_EXPSIG;
+	} else if (!strncmp (status, "EXPKEYSIG ", 10)) {
+		signer = gpg_ctx_add_signer (gpg, GMIME_SIGNER_STATUS_ERROR, status + 10);
+		signer->errors |= GMIME_SIGNER_ERROR_EXPKEYSIG;
+	} else if (!strncmp (status, "REVKEYSIG ", 10)) {
+		signer = gpg_ctx_add_signer (gpg, GMIME_SIGNER_STATUS_ERROR, status + 10);
+		signer->errors |= GMIME_SIGNER_ERROR_REVKEYSIG;
 	} else if (!strncmp (status, "ERRSIG ", 7)) {
 		/* Note: NO_PUBKEY often comes after an ERRSIG */
 		status += 7;
@@ -865,36 +890,50 @@ gpg_ctx_parse_signer_info (struct _GpgCtx *gpg, char *status)
 		/* get the key id of the signer */
 		status = next_token (status, &signer->keyid);
 		
-		/* skip the pubkey_algo */
-		status = next_token (status, NULL);
+		/* the second token is the public-key algorithm id */
+		signer->pubkey_algo = strtoul (status, &inend, 10);
+		if (inend == status || *inend != ' ')
+			return;
 		
-		/* skip the digest_algo */
-		status = next_token (status, NULL);
+		status = inend + 1;
 		
-		/* skip the class */
-		status = next_token (status, NULL);
+		/* the third token is the hash algorithm id */
+		signer->hash_algo = strtoul (status, &inend, 10);
+		if (inend == status || *inend != ' ')
+			return;
 		
-		/* get the signature expiration date (or 0 for never) */
-		signer->sig_expires = strtoul (status, NULL, 10);
-		status = next_token (status, NULL);
+		status = inend + 1;
 		
-		/* the last token is the 'rc' which we don't care about */
+		/* the fourth token is the signature class */
+		signer->sig_class = strtoul (status, &inend, 10);
+		if (inend == status || *inend != ' ') {
+			signer->sig_class = 0;
+			return;
+		}
+		
+		status = inend + 1;
+		
+		/* the fifth token is the signature expiration date (or 0 for never) */
+		signer->sig_expires = strtoul (status, &inend, 10);
+		if (inend == status || *inend != ' ') {
+			signer->sig_expires = 0;
+			return;
+		}
+		
+		status = inend + 1;
+		
+		/* the sixth token is the return code */
+		switch (strtol (status, NULL, 10)) {
+		case 4: signer->errors |= GMIME_SIGNER_ERROR_UNSUPP_ALGO; break;
+		case 9: signer->errors |= GMIME_SIGNER_ERROR_NO_PUBKEY; break;
+		default: break;
+		}
 	} else if (!strncmp (status, "NO_PUBKEY ", 10)) {
 		/* the only token is the keyid, but we've already got it */
 		gpg->signer->errors |= GMIME_SIGNER_ERROR_NO_PUBKEY;
-	} else if (!strncmp (status, "EXPSIG", 6)) {
-		/* FIXME: see what else we can glean from this... */
-		gpg->signer->errors |= GMIME_SIGNER_ERROR_EXPSIG;
-	} else if (!strncmp (status, "EXPKEYSIG", 9)) {
-		gpg->signer->errors |= GMIME_SIGNER_ERROR_EXPKEYSIG;
-	} else if (!strncmp (status, "REVKEYSIG", 9)) {
-		gpg->signer->errors |= GMIME_SIGNER_ERROR_REVKEYSIG;
 	} else if (!strncmp (status, "VALIDSIG ", 9)) {
-		char *inend;
-		
-		status += 9;
-		
 		signer = gpg->signer;
+		status += 9;
 		
 		/* the first token is the fingerprint */
 		status = next_token (status, &signer->fingerprint);
@@ -921,27 +960,39 @@ gpg_ctx_parse_signer_info (struct _GpgCtx *gpg, char *status)
 		status = inend + 1;
 		
 		/* the fifth token is the signature version */
-		status = next_token (status, NULL);
+		signer->sig_ver = strtoul (status, &inend, 10);
+		if (inend == status || *inend != ' ') {
+			signer->sig_ver = 0;
+			return;
+		}
+		
+		status = inend + 1;
 		
 		/* the sixth token is a reserved numeric value (ignore for now) */
 		status = next_token (status, NULL);
 		
 		/* the seventh token is the public-key algorithm id */
-		signer->pubkey_algo = gpg_pubkey_algo (strtoul (status, &inend, 10));
+		signer->pubkey_algo = strtoul (status, &inend, 10);
 		if (inend == status || *inend != ' ')
 			return;
 		
 		status = inend + 1;
 		
 		/* the eighth token is the hash algorithm id */
-		signer->hash_algo = gpg_hash_algo (strtoul (status, &inend, 10));
+		signer->hash_algo = strtoul (status, &inend, 10);
 		if (inend == status || *inend != ' ')
 			return;
 		
 		status = inend + 1;
 		
 		/* the nineth token is the signature class */
-		status = next_token (status, NULL);
+		signer->sig_class = strtoul (status, &inend, 10);
+		if (inend == status || *inend != ' ') {
+			signer->sig_class = 0;
+			return;
+		}
+		
+		status = inend + 1;
 		
 		/* the rest is the primary key fingerprint */
 	} else if (!strncmp (status, "TRUST_", 6)) {
@@ -1160,6 +1211,9 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 		
 		return -1;
 	} else {
+		GMimeCryptoRecipient *recipient;
+		char *inend;
+		
 		switch (gpg->mode) {
 		case GPG_CTX_MODE_SIGN:
 			if (strncmp (status, "SIG_CREATED ", 12) != 0)
@@ -1174,7 +1228,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 			status = next_token (status, NULL);
 			
 			/* this token is the hash algorithm used */
-			gpg->hash = gpg_hash_algo (strtoul (status, NULL, 10));
+			gpg->hash = strtoul (status, NULL, 10);
 			break;
 		case GPG_CTX_MODE_VERIFY:
 			gpg_ctx_parse_signer_info (gpg, status);
@@ -1194,8 +1248,48 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 		case GPG_CTX_MODE_DECRYPT:
 			if (!strncmp (status, "BEGIN_DECRYPTION", 16)) {
 				/* nothing to do... but we know to expect data on stdout soon */
+			} else if (!strncmp (status, "DECRYPTION_INFO ", 16)) {
+				/* new feature added in gnupg-2.1.x which gives mdc and cipher algorithms used */
+				status += 16;
+				
+				/* first token is the mdc algorithm (or 0 if not used) */
+				gpg->hash = strtoul (status, &inend, 10);
+				if (inend == status || *inend != ' ')
+					return;
+				
+				status = inend + 1;
+				
+				/* second token is the cipher algorithm */
+				gpg->cipher = strtoul (status, &inend, 10);
+			} else if (!strncmp (status, "DECRYPTION_OKAY", 15)) {
+				/* nothing to do... but good to know gpg decrypted the data successfully */
+			} else if (!strncmp (status, "DECRYPTION_FAILED", 17)) {
+				/* nothing to do... but we know gpg failed to decrypt :-( */
 			} else if (!strncmp (status, "END_DECRYPTION", 14)) {
 				/* nothing to do, but we know we're done */
+			} else if (!strncmp (status, "ENC_TO ", 7)) {
+				/* parse the recipient info */
+				recipient = g_mime_crypto_recipient_new ();
+				gpg->enc_to->next = recipient;
+				gpg->enc_to = recipient;
+				
+				status += 7;
+				
+				/* first token is the recipient's keyid */
+				status = next_token (status, &recipient->keyid);
+				
+				/* second token is the recipient's pubkey algo */
+				recipient->pubkey_algo = strtoul (status, &inend, 10);
+				if (inend == status || *inend != ' ')
+					return;
+				
+				status = inend + 1;
+				
+				/* third token is a dummy value which is always '0' */
+			} else if (!strncmp (status, "GOODMDC", 7)) {
+				/* nothing to do... we'll grab the MDC used in DECRYPTION_INFO */
+			} else if (!strncmp (status, "BADMDC", 6)) {
+				/* nothing to do, this will only be sent after DECRYPTION_FAILED */
 			} else {
 				gpg_ctx_parse_signer_info (gpg, status);
 			}
@@ -1845,12 +1939,12 @@ gpg_encrypt (GMimeCryptoContext *context, gboolean sign, const char *userid,
 }
 
 
-static GMimeSignatureValidity *
+static GMimeDecryptionResult *
 gpg_decrypt (GMimeCryptoContext *context, GMimeStream *istream,
 	     GMimeStream *ostream, GError **err)
 {
 	GMimeGpgContext *ctx = (GMimeGpgContext *) context;
-	GMimeSignatureValidity *validity;
+	GMimeDecryptionResult *result;
 	const char *diagnostics;
 	struct _GpgCtx *gpg;
 	int save;
@@ -1889,16 +1983,20 @@ gpg_decrypt (GMimeCryptoContext *context, GMimeStream *istream,
 		return NULL;
 	}
 	
+	result = g_mime_decryption_result_new ();
+	result->validity = g_mime_signature_validity_new ();
 	diagnostics = gpg_ctx_get_diagnostics (gpg);
-	
-	validity = g_mime_signature_validity_new ();
-	g_mime_signature_validity_set_details (validity, diagnostics);
-	validity->signers = gpg->signers;
+	g_mime_signature_validity_set_details (result->validity, diagnostics);
+	result->validity->signers = gpg->signers;
+	result->recipients = gpg->encrypted_to;
+	result->cipher = gpg->cipher;
+	result->mdc = gpg->hash;
+	gpg->encrypted_to = NULL;
 	gpg->signers = NULL;
 	
 	gpg_ctx_free (gpg);
 	
-	return validity;
+	return result;
 }
 
 static int
