@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2011 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -23,7 +23,7 @@
 #define __GMIME_MULTIPART_SIGNED_H__
 
 #include <gmime/gmime-multipart.h>
-#include <gmime/gmime-cipher-context.h>
+#include <gmime/gmime-crypto-context.h>
 
 G_BEGIN_DECLS
 
@@ -46,16 +46,12 @@ enum {
 /**
  * GMimeMultipartSigned:
  * @parent_object: parent #GMimeMultipart
- * @protocol: signature protocol string
- * @micalg: signature hash
  *
  * A multipart/signed MIME part.
  **/
 struct _GMimeMultipartSigned {
 	GMimeMultipart parent_object;
 	
-	char *protocol;
-	char *micalg;
 };
 
 struct _GMimeMultipartSignedClass {
@@ -69,11 +65,11 @@ GType g_mime_multipart_signed_get_type (void);
 GMimeMultipartSigned *g_mime_multipart_signed_new (void);
 
 int g_mime_multipart_signed_sign (GMimeMultipartSigned *mps, GMimeObject *content,
-				  GMimeCipherContext *ctx, const char *userid,
-				  GMimeCipherHash hash, GError **err);
+				  GMimeCryptoContext *ctx, const char *userid,
+				  GMimeCryptoHash hash, GError **err);
 
 GMimeSignatureValidity *g_mime_multipart_signed_verify (GMimeMultipartSigned *mps,
-							GMimeCipherContext *ctx,
+							GMimeCryptoContext *ctx,
 							GError **err);
 
 G_END_DECLS
