@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2011 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
 
 #include <gmime/gmime.h>
 
-#ifndef G_OS_WIN32
+#if !defined (G_OS_WIN32) || defined (__MINGW32__)
 #define ENABLE_ZENTIMER
 #include "zentimer.h"
 #endif
