@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2011 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -39,22 +39,39 @@ extern int verbose;
 #define v(x) if (verbose > 3) x
 
 static gboolean
-request_passwd (GMimeCipherContext *ctx, const char *user_id, const char *prompt_ctx, gboolean reprompt, GMimeStream *response, GError **err)
+request_passwd (GMimeCryptoContext *ctx, const char *user_id, const char *prompt_ctx, gboolean reprompt, GMimeStream *response, GError **err)
 {
 	g_mime_stream_write_string (response, "no.secret\n");
 	
 	return TRUE;
 }
 
+static GMimeSignerStatus
+get_sig_status (GMimeSigner *signers)
+{
+	GMimeSignerStatus status = GMIME_SIGNER_STATUS_GOOD;
+	GMimeSigner *signer = signers;
+	
+	if (signers == NULL)
+		return GMIME_SIGNER_STATUS_ERROR;
+	
+	while (signer != NULL) {
+		status = MAX (status, signer->status);
+		signer = signer->next;
+	}
+	
+	return status;
+}
+
 static void
-test_sign (GMimeCipherContext *ctx, GMimeStream *cleartext, GMimeStream *ciphertext)
+test_sign (GMimeCryptoContext *ctx, GMimeStream *cleartext, GMimeStream *ciphertext)
 {
 	GError *err = NULL;
 	Exception *ex;
 	int rv;
 	
-	rv = g_mime_cipher_context_sign (ctx, "no.user@no.domain",
-					 GMIME_CIPHER_HASH_SHA256,
+	rv = g_mime_crypto_context_sign (ctx, "no.user@no.domain",
+					 GMIME_CRYPTO_HASH_SHA256,
 					 cleartext, ciphertext, &err);
 	
 	if (rv == -1 || err != NULL) {
@@ -64,19 +81,19 @@ test_sign (GMimeCipherContext *ctx, GMimeStream *cleartext, GMimeStream *ciphert
 	}
 	
 	v(fprintf (stderr, "signature (%s):\n%.*s\n",
-		   g_mime_cipher_context_hash_name (ctx, rv),
+		   g_mime_crypto_context_hash_name (ctx, rv),
 		   GMIME_STREAM_MEM (ciphertext)->buffer->len,
 		   GMIME_STREAM_MEM (ciphertext)->buffer->data));
 }
 
 static void
-test_verify (GMimeCipherContext *ctx, GMimeStream *cleartext, GMimeStream *ciphertext)
+test_verify (GMimeCryptoContext *ctx, GMimeStream *cleartext, GMimeStream *ciphertext)
 {
 	GMimeSignatureValidity *validity;
 	GError *err = NULL;
 	Exception *ex;
 	
-	validity = g_mime_cipher_context_verify (ctx, GMIME_CIPHER_HASH_DEFAULT,
+	validity = g_mime_crypto_context_verify (ctx, GMIME_CRYPTO_HASH_DEFAULT,
 						 cleartext, ciphertext, &err);
 	
 	if (validity == NULL) {
@@ -85,7 +102,7 @@ test_verify (GMimeCipherContext *ctx, GMimeStream *cleartext, GMimeStream *ciphe
 		throw (ex);
 	}
 	
-	if (validity->status != GMIME_SIGNATURE_STATUS_GOOD) {
+	if (get_sig_status (validity->signers) != GMIME_SIGNER_STATUS_GOOD) {
 		g_mime_signature_validity_free (validity);
 		throw (exception_new ("signature BAD"));
 	}
@@ -94,7 +111,7 @@ test_verify (GMimeCipherContext *ctx, GMimeStream *cleartext, GMimeStream *ciphe
 }
 
 static void
-test_encrypt (GMimeCipherContext *ctx, gboolean sign, GMimeStream *cleartext, GMimeStream *ciphertext)
+test_encrypt (GMimeCryptoContext *ctx, gboolean sign, GMimeStream *cleartext, GMimeStream *ciphertext)
 {
 	GPtrArray *recipients;
 	GError *err = NULL;
@@ -103,8 +120,8 @@ test_encrypt (GMimeCipherContext *ctx, gboolean sign, GMimeStream *cleartext, GM
 	recipients = g_ptr_array_new ();
 	g_ptr_array_add (recipients, "no.user@no.domain");
 	
-	g_mime_cipher_context_encrypt (ctx, sign, "no.user@no.domain", recipients,
-				       cleartext, ciphertext, &err);
+	g_mime_crypto_context_encrypt (ctx, sign, "no.user@no.domain", GMIME_CRYPTO_HASH_SHA256,
+				       recipients, cleartext, ciphertext, &err);
 	
 	g_ptr_array_free (recipients, TRUE);
 	
@@ -120,9 +137,9 @@ test_encrypt (GMimeCipherContext *ctx, gboolean sign, GMimeStream *cleartext, GM
 }
 
 static void
-test_decrypt (GMimeCipherContext *ctx, gboolean sign, GMimeStream *cleartext, GMimeStream *ciphertext)
+test_decrypt (GMimeCryptoContext *ctx, gboolean sign, GMimeStream *cleartext, GMimeStream *ciphertext)
 {
-	GMimeSignatureValidity *sv;
+	GMimeDecryptionResult *result;
 	Exception *ex = NULL;
 	GMimeStream *stream;
 	GError *err = NULL;
@@ -130,7 +147,7 @@ test_decrypt (GMimeCipherContext *ctx, gboolean sign, GMimeStream *cleartext, GM
 	
 	stream = g_mime_stream_mem_new ();
 	
-	if (!(sv = g_mime_cipher_context_decrypt (ctx, ciphertext, stream, &err))) {
+	if (!(result = g_mime_crypto_context_decrypt (ctx, ciphertext, stream, &err))) {
 		g_object_unref (stream);
 		ex = exception_new ("%s", err->message);
 		g_error_free (err);
@@ -138,14 +155,14 @@ test_decrypt (GMimeCipherContext *ctx, gboolean sign, GMimeStream *cleartext, GM
 	}
 	
 	if (sign) {
-		if (sv->status != GMIME_SIGNATURE_STATUS_GOOD)
+		if (!result->validity || get_sig_status (result->validity->signers) != GMIME_SIGNER_STATUS_GOOD)
 			ex = exception_new ("expected GOOD signature");
 	} else {
-		if (sv->status != GMIME_SIGNATURE_STATUS_NONE)
+		if (result->validity && result->validity->signers != NULL)
 			ex = exception_new ("unexpected signature");
 	}
 	
-	g_mime_signature_validity_free (sv);
+	g_mime_decryption_result_free (result);
 	
 	if (ex != NULL) {
 		g_object_unref (stream);
@@ -165,7 +182,7 @@ test_decrypt (GMimeCipherContext *ctx, gboolean sign, GMimeStream *cleartext, GM
 }
 
 static void
-test_export (GMimeCipherContext *ctx, const char *path)
+test_export (GMimeCryptoContext *ctx, const char *path)
 {
 	GMimeStream *istream, *ostream;
 	register const char *inptr;
@@ -190,7 +207,7 @@ test_export (GMimeCipherContext *ctx, const char *path)
 	
 	ostream = g_mime_stream_mem_new ();
 	
-	g_mime_cipher_context_export_keys (ctx, keys, ostream, &err);
+	g_mime_crypto_context_export_keys (ctx, keys, ostream, &err);
 	g_ptr_array_free (keys, TRUE);
 	if (err != NULL) {
 		ex = exception_new ("%s", err->message);
@@ -229,7 +246,7 @@ test_export (GMimeCipherContext *ctx, const char *path)
 }
 
 static void
-import_key (GMimeCipherContext *ctx, const char *path)
+import_key (GMimeCryptoContext *ctx, const char *path)
 {
 	GMimeStream *stream;
 	GError *err = NULL;
@@ -240,7 +257,7 @@ import_key (GMimeCipherContext *ctx, const char *path)
 		throw (exception_new ("open() failed: %s", g_strerror (errno)));
 	
 	stream = g_mime_stream_fs_new (fd);
-	g_mime_cipher_context_import_keys (ctx, stream, &err);
+	g_mime_crypto_context_import_keys (ctx, stream, &err);
 	g_object_unref (stream);
 	
 	if (err != NULL) {
@@ -255,7 +272,7 @@ int main (int argc, char **argv)
 {
 	const char *datadir = "data/pgp";
 	GMimeStream *istream, *ostream;
-	GMimeCipherContext *ctx;
+	GMimeCryptoContext *ctx;
 	const char *what;
 	struct stat st;
 	char *key;
@@ -281,7 +298,7 @@ int main (int argc, char **argv)
 	if (i < argc && (stat (datadir, &st) == -1 || !S_ISDIR (st.st_mode)))
 		return EXIT_FAILURE;
 	
-	testsuite_start ("GnuPG cipher context");
+	testsuite_start ("GnuPG crypto context");
 	
 	ctx = g_mime_gpg_context_new (request_passwd, "/usr/bin/gpg");
 	g_mime_gpg_context_set_always_trust ((GMimeGpgContext *) ctx, TRUE);
